@@ -3,10 +3,17 @@
 #include <fcntl.h>
 #include <unistd.h>
 #include <memory.h>
+#include <errno.h>
+
+#include "verbose.h"
 
 bool disk_dev_open(disk_dev_t *dev, const char *path)
 {
 	dev->fd = open(path, O_RDWR|O_DIRECT);
+	if (dev->fd < 0) {
+		INFO("Failed to open device %s with write permission, retrying without", path);
+		dev->fd = open(path, O_RDONLY|O_DIRECT);
+	}
 	return dev->fd >= 0;
 }
 
@@ -33,6 +40,8 @@ ssize_t disk_dev_read(disk_dev_t *dev, uint64_t offset_bytes, uint32_t len_bytes
 		return ret;
 	} else {
 		// ret < 0, i.e. error
+		INFO("Error reading from disk, offset=%lu len=%u errno=%d (%s)",
+			offset_bytes, len_bytes, errno, strerror(errno));
 		io_res->data = DATA_NONE;
 		io_res->error = ERROR_UNCORRECTED;
 		io_res->sense_len = 0;
@@ -72,14 +81,17 @@ ssize_t disk_dev_write(disk_dev_t *dev, uint64_t offset_bytes, uint32_t len_byte
 
 void disk_dev_cdb_in(disk_dev_t *dev, unsigned char *cdb, unsigned cdb_len, unsigned char *buf, unsigned buf_size, unsigned *buf_read, unsigned char *sense, unsigned sense_size, unsigned *sense_read, io_result_t *io_res)
 {
-	memset(&io_res, 0, sizeof(*io_res));
-	io_res->data = DATA_NONE;
-	io_res->error = ERROR_UNKNOWN;
-}
+	(void)sense_size;
+	(void)sense;
+	(void)buf_size;
+	(void)buf;
+	(void)cdb_len;
+	(void)cdb;
+	(void)dev;
 
-void disk_dev_cdb_in(disk_dev_t *dev, unsigned char *cdb, unsigned cdb_len, unsigned char *buf, unsigned buf_size, unsigned *buf_read, unsigned char *sense, unsigned sense_size, unsigned *sense_read, io_result_t *io_res)
-{
+	*sense_read = 0;
+	*buf_read = 0;
 	memset(&io_res, 0, sizeof(*io_res));
 	io_res->data = DATA_NONE;
-	io_res->error = ERROR_UNKNOWN;
+	io_res->error = ERROR_NONE;
 }
