@@ -23,6 +23,9 @@
 #include "compiler.h"
 #include "cli.h"
 
+#include "progressbar/include/progressbar.h"
+#include "hdrhistogram/src/hdr_histogram.h"
+
 #include <stdio.h>
 #include <signal.h>
 #include <inttypes.h>
@@ -33,6 +36,7 @@
 #include <errno.h>
 
 static disk_t disk;
+static progressbar *bar;
 
 typedef struct options_t options_t;
 struct options_t {
@@ -69,8 +73,9 @@ static int usage(void) {
 
 void report_progress(disk_t * UNUSED(disk), int progress_part, int progress_full)
 {
-	printf("Progress: %4.1f%%\r", (float)progress_part * 100.0 / (float)progress_full);
-	fflush(stdout);
+	if (bar == NULL)
+		bar = progressbar_new("Disk scan", progress_full);
+	progressbar_update(bar, progress_part);
 }
 
 void report_scan_success(disk_t *UNUSED(disk), uint64_t UNUSED(offset_bytes), uint64_t UNUSED(data_size), uint64_t UNUSED(time))
@@ -145,20 +150,15 @@ static void print_latency(latency_t *latency_graph, unsigned latency_graph_len)
 
 void report_scan_done(disk_t *pdisk)
 {
-	unsigned hist_idx;
-	
-	printf("Access time histogram:\n");
-	for (hist_idx = 0; hist_idx < ARRAY_SIZE(pdisk->histogram); hist_idx++)
-	{
-		if (hist_idx != ARRAY_SIZE(pdisk->histogram)-1)
-			printf("%8" PRIu64 ": %" PRIu64 "\n", histogram_time[hist_idx].top_val, pdisk->histogram[hist_idx]);
-		else
-			printf("%8s: %" PRIu64 "\n", "above that", pdisk->histogram[hist_idx]);
-	}
+	progressbar_finish(bar);
 
+	printf("\nAccess time histogram:\n");
+	hdr_percentiles_print(pdisk->histogram, stdout, 5, 1000.0, CLASSIC); // Print msecs
+
+	printf("\nLatency graph:\n");
 	print_latency(pdisk->latency_graph, pdisk->latency_graph_len);
 
-	printf("Conclusion: %s\n", conclusion_to_str(pdisk->conclusion));
+	printf("\nConclusion: %s\n", conclusion_to_str(pdisk->conclusion));
 }
 
 static unsigned str_to_scan_size(const char *str)
