@@ -276,23 +276,23 @@ int disk_open(disk_t *disk, const char *path, int fix, unsigned latency_graph_le
 
 	INFO("Validating path %s", path);
 	if (access(path, F_OK)) {
-		ERROR("Disk path %s does not exist: %m", path);
+		ERROR("Disk path %s does not exist, errno=%d: %s", path, errno, strerror(errno));
 		return 1;
 	}
 
 	const int access_mode_flag = fix ? R_OK|W_OK : R_OK;
 	if (access(path, access_mode_flag)) {
-		ERROR("Disk path %s is inaccessible: %m", path);
+		ERROR("Disk path %s is inaccessible, errno=%d: %s", path, errno, strerror(errno));
 		return 1;
 	}
 
 	if (!disk_dev_open(&disk->dev, path)) {
-		ERROR("Failed to open path %s: %m", path);
+		ERROR("Failed to open path %s, errno=%d: %s", path, errno, strerror(errno));
 		return 1;
 	}
 
 	if (disk_dev_read_cap(&disk->dev, &disk->num_bytes, &disk->sector_size) < 0) {
-		ERROR("Can't get block device size information for path %s: %m", path);
+		ERROR("Can't get block device size information for path %s, errno=%d: %s", path, errno, strerror(errno));
 		goto Error;
 	}
 
@@ -314,7 +314,7 @@ int disk_open(disk_t *disk, const char *path, int fix, unsigned latency_graph_le
 #endif
 
 	if (disk_dev_identify(&disk->dev, disk->vendor, disk->model, disk->fw_rev, disk->serial, &disk->is_ata, disk->ata_buf, &disk->ata_buf_len) < 0) {
-		ERROR("Can't identify disk for path %s: %m", path);
+		ERROR("Can't identify disk for path %s, errno=%d: %s", path, errno, strerror(errno));
 		goto Error;
 	}
 
@@ -469,7 +469,7 @@ static bool disk_scan_part(disk_t *disk, uint64_t offset, void *data, int data_s
 	// Handle error or incomplete data
 	if (io_res.data != DATA_FULL || io_res.error != ERROR_NONE) {
 		int s_errno = errno;
-		ERROR("Error when reading at offset %" PRIu64 " size %d read %zd: %m", offset, data_size, ret);
+		ERROR("Error when reading at offset %" PRIu64 " size %d read %zd, errno=%d: %s", offset, data_size, ret, errno, strerror(errno));
 		ERROR("Details: error=%s data=%s %02X/%02X/%02X", error_to_str(io_res.error), data_to_str(io_res.data),
 				io_res.info.sense_key, io_res.info.asc, io_res.info.ascq);
 		report_scan_error(disk, offset, data_size, t);
@@ -505,10 +505,31 @@ static bool disk_scan_part(disk_t *disk, uint64_t offset, void *data, int data_s
 	}
 
 	if (disk->fix && (t_msec > 3000 || error)) {
-		INFO("Fixing region by rewriting, offset=%"PRIu64" size=%d", offset, data_size);
-		ret = disk_dev_write(&disk->dev, offset, data_size, data, &io_res);
-		if (ret != data_size) {
-			ERROR("Error while attempting to rewrite the data! ret=%zd errno=%d: %m", ret, errno);
+		if (io_res.error != ERROR_UNCORRECTED) {
+			INFO("Fixing region by rewriting, offset=%"PRIu64" size=%d", offset, data_size);
+			ret = disk_dev_write(&disk->dev, offset, data_size, data, &io_res);
+			if (ret != data_size) {
+				ERROR("Error while attempting to rewrite the data! ret=%zd errno=%d: %s", ret, errno, strerror(errno));
+			}
+		} else {
+			// When we correct uncorrectable errors we want to zero it out, this should reduce any confusion later on when the data is read
+			unsigned fix_offset = 0;
+			int fix_size = 4096;
+
+			if (data_size < fix_size)
+				fix_size = data_size;
+
+			for (; data_size >= (int)(fix_offset + fix_size); fix_offset += fix_size) {
+				disk_dev_read(&disk->dev, offset+fix_offset, fix_size, data, &io_res);
+				if (io_res.error == ERROR_UNCORRECTED) {
+					INFO("Fixing uncorrectable region by writing zeros, offset=%"PRIu64" size=%d", offset+fix_offset, fix_size);
+					memset(data, 0, fix_size);
+					ret = disk_dev_write(&disk->dev, offset+fix_offset, fix_size, data, &io_res);
+					if (ret != data_size) {
+						ERROR("Error while attempting to overwrite uncorrectable data! ret=%zd errno=%d: %s", ret, errno, strerror(errno));
+					}
+				}
+			}
 		}
 	}
 
@@ -597,6 +618,9 @@ static void progress_calc(disk_t *disk, struct scan_state *state, uint64_t add)
 static bool disk_scan_latency_stride(disk_t *disk, struct scan_state *state, uint64_t base_offset, uint64_t data_size, uint32_t *scan_order)
 {
 	unsigned i;
+	uint64_t stride_end = base_offset + state->latency_stride * disk->sector_size;
+	if (stride_end > disk->num_bytes)
+		stride_end = disk->num_bytes;
 
 	for (i = 0; disk->run && scan_order[i] != UINT32_MAX; i++) {
 		uint64_t offset = base_offset + scan_order[i];
@@ -604,14 +628,13 @@ static bool disk_scan_latency_stride(disk_t *disk, struct scan_state *state, uin
 		progress_calc(disk, state, data_size);
 
 		VVVERBOSE("Scanning at offset %"PRIu64" index %u", offset, i);
-		uint64_t stride_end = base_offset + state->latency_stride * disk->sector_size;
-		if (stride_end > disk->num_bytes)
-			stride_end = disk->num_bytes;
-		uint64_t remainder = stride_end - offset;
-		if (remainder < data_size) {
+		int64_t remainder = stride_end - offset;
+		if (remainder < (int64_t)data_size) {
 			data_size = remainder;
 			VERBOSE("Last part scanning size %"PRIu64, data_size);
 		}
+		if (offset > disk->num_bytes || (offset+remainder) > disk->num_bytes)
+			continue;
 		if (!disk_scan_part(disk, offset, state->data, data_size, state))
 			return false;
 	}
@@ -675,7 +698,7 @@ int disk_scan(disk_t *disk, enum scan_mode mode, unsigned data_size)
 	VVVERBOSE("Using buffer of size %d", data_size);
 
 	if (data == NULL) {
-		ERROR("Failed to allocate data buffer: %m");
+		ERROR("Failed to allocate data buffer, errno=%d: %s", errno, strerror(errno));
 		result = 1;
 		goto Exit;
 	}
